from getpass import getpass
from pollenisatorgui.core.components import utils
import psutil


def findDc(apiclient, domain=None, ask_if_fail=True, graphical=False):
    search = {"type":"computer", "infos.is_dc":True}
    if domain is not None:
        search["domain"] = domain
    dc_info = apiclient.find("computers", search, False)
    if dc_info is None:
        if not ask_if_fail:
            return False, "DC not known"
        if graphical:
            from pollenisatorgui.core.application.dialogs.ChildDialogAskText import ChildDialogAskText
            dialog = ChildDialogAskText(None, "DC not known, give me its IP if you know it", multiline=False)
            dialog.app.wait_window(dialog.app)
            dc = dialog.rvalue
        else:
            dc = input("DC not found, give me its IP if you know it:")
    else:
        dc = dc_info.get("ip")
    if dc is None or dc == "":
        return False, "DC not known"
    return True, dc
    

def checkPath(appnames):
    if isinstance(appnames, str):
        appnames = [appnames]
    for appname in appnames:
        path = utils.which_expand_alias(appname)
        if path is not None:
            return True, path
    return False, "App name not found, create an alias or install it. ("+", ".join(appnames)+" were tested)"
    
def getNICs(graphical=False):
    addrs = psutil.net_if_addrs()
    addrs_keys = list(addrs.keys())
    if graphical:
        from pollenisatorgui.core.application.dialogs.ChildDialogCombo import ChildDialogCombo

        dialog = ChildDialogCombo(None, addrs_keys, displayMsg="Choose your ethernet device to listen on")
        dialog.app.wait_window(dialog.app)
        if dialog.rvalue is None:
            raise ValueError("No ethernet device chosen")
        eth = dialog.rvalue
    else:
        
        print("Choose your ethernet device to listen on")
        for i, ethitem in enumerate(addrs_keys):
            print(str(i+1)+". "+ethitem)

        eth = input("Choose your ethernet device to listen on (type its number):")
        if eth is None:
            raise ValueError("No ethernet device chosen")
        try:
            eth = addrs_keys[int(eth)-1]
        except ValueError:
            raise ValueError("Wrong number given")
    if eth is None or eth == "":
        raise ValueError("No ethernet device chosen")
        
    return eth


def ask_text(graphical, title, default, secret=False):
    if graphical:
        from pollenisatorgui.core.application.dialogs.ChildDialogAskText import ChildDialogAskText
        dialog = ChildDialogAskText(None, title, default, secret)
        return dialog.rvalue
    else:
        if secret:
            res = getpass(title+ " (or default to :"+str(default)+")")
        res = input(title+" (or default to :"+str(default)+")").strip()
        if res == "" or res is None:
            return default
        return res


import subprocess
import pollenisatorgui.core.components.utils as utils
import os
from pollenisatorgui.core.components.apiclient import APIClient

from pollenisatorgui.scripts.lan.utils import ask_text, checkPath, getNICs


def main(apiclient, appli, **kwargs):
    res, path = checkPath(["mitm6", "mitm6.py"])
    if not res:
        return False, path
    APIClient.setInstance(apiclient)
    smb_signing_list = apiclient.find("computers", {}, True)
    export_dir = utils.getExportDir()
    domains = set()
    for computer in smb_signing_list:
        domain = computer.get("domain", "")
        if domain.strip() != "":
            domains.add(domain)
    domains = list(domains)
    relaying_loot_path = os.path.join(export_dir, "loot_relay")
    try:
        os.makedirs(relaying_loot_path)
    except:
        pass
    relaying_loot_path = os.path.join(relaying_loot_path, "hashes-mitm6.log")
    device = getNICs(appli is not None)
    if device is None or device.strip() == "":
        return False, "No device selected"
    domain = ""
    if len(domains) == 0:
        domain = ask_text(appli is not None, "No domain found, enter domain name :", "")
    elif len(domains) == 1:
        domain = domains[0]
    else:
        if appli:
            from pollenisatorgui.core.application.dialogs.ChildDialogCombo import ChildDialogCombo
            
            dialog = ChildDialogCombo(None, domains, displayMsg="Choose target domain")
            dialog.app.wait_window(dialog.app)
            domain = dialog.rvalue
        else:
            print("Choose target domain")
            for i, domainitem in enumerate(domains):
                print(str(i+1)+". "+domainitem)
            domain = input("Choose target domain (type its number):")
            try:
                domain = domains[int(domain)-1]
            except ValueError:
                return False, "Wrong number given"
    if domain is None or domain == "":
        return False, "No domain choosen"
    cmd = f"mitm6 -i {device} -d {domain}"
    if os.geteuid() != 0:
        cmd = "sudo "+cmd
    if appli:
        appli.launch_in_terminal(kwargs.get("default_target",None), "mitm6", f"{cmd}", use_pollex=False)
    else:
        subprocess.run(f"{cmd}", shell=True)
    return True, f"Listening ntlmrelayx with mitm6 opened, loot directory is here:"+str(relaying_loot_path)+"\n"
    



#!/usr/bin/python3
# -*- coding: utf-8 -*-
"""
@author: Fabien Barré for AlgoSecure
# Date: 12/10/2022
# Major version released: 09/2019
# @version: 2.6.0
"""

import time
import os
import sys
import signal
import requests
from pollenisatorgui.core.components.apiclient import APIClient
import pollenisatorgui.core.components.utils as utils
import threading
from getpass import getpass
from pollenisatorgui.core.components.logger_config import logger

event_obj = threading.Event()

class GracefulKiller:
    """
    Signal handler to shut down properly.

    Attributes:
        kill_now: a boolean that can checked to know that it's time to stop.
    """
    kill_now = False

    def __init__(self, app):
        """
        Constructor. Hook the signals SIGINT and SIGTERM to method exitGracefully

        Args:
            app: The appli object to stop
        """
        signal.signal(signal.SIGINT, self.exitGracefully)
        signal.signal(signal.SIGTERM, self.exitGracefully)
        # signal.signal(signal.SIGPIPE, signal.SIG_DFL)
        self.app = app

    def exitGracefully(self, _signum, _frame):
        """
        Set the kill_now class attributes to True. Call the onClosing function of the application given at init.

        Args:
            signum: not used
            frame: not used
        """
        #import traceback
        #traceback.print_stack(_frame)
        self.app.onClosing()
        event_obj.set()
        self.kill_now = True


#######################################
############## MAIN ###################
#######################################

def get_local_git_version():
    return "v2.7.1"

def get_latest_github_version():
    return None
    try:
        # Make a GET request to GitHub's releases endpoint
        url = f"https://api.github.com/repos/fbarre96/PollenisatorGUI/releases/latest"
        response = requests.get(url)
        if response.status_code == 200:
            latest_version = response.json()["tag_name"]
            return latest_version
        else:
            print("Error fetching latest version from GitHub:", response.text)
            return None
    except Exception as e:
        print("Error fetching latest version from GitHub:", e)
        return None

def checkForUpdates():
    latest = get_latest_github_version()
    local = get_local_git_version()
    if local is None or latest is None:
        return False
    if local != latest:
        print(f"New version available: {latest} (current: {local})")
        return True
    return False

def update():
    try:
        # Use pipx to install the latest version of the package
        subprocess.run(['pipx', 'upgrade', 'pollenisator-gui'])
        print("Package updated successfully! Restart the application to use the latest version.")
        sys.exit(0)
    except Exception as e:
        print("Error updating package:", e)

def consoleConnect(force=False, askPentest=True):
    apiclient = APIClient.getInstance()
    abandon = False
    if force:
        apiclient.disconnect()
    while (not apiclient.tryConnection(force=force) or not apiclient.isConnected()) and not abandon:
        success = promptForConnection() is None
    if apiclient.isConnected() and askPentest:
        promptForPentest()
    return not abandon

def promptForConnection():

    clientCfg = utils.loadClientConfig()
    host = clientCfg.get("host", "")
    port = clientCfg.get("port", "5000")
    https = str(clientCfg.get("https", "True")).title() == "True"

    host_result = input(f'Server hostname (defaut {host}):')
    if host_result.strip() == "":
        host_result = host
    port_result = input(f'Server Port (default  {port}):')
    if port_result.strip() == "":
        port_result = port
    https_result = input(f'Use HTTPS (y/N)?').strip() == 'y'

    apiclient = APIClient.getInstance()
    apiclient.reinitConnection()
    res = apiclient.tryConnection({"host":host_result, "port":port_result, "https":https_result})
    if res:
        username_result = input(f'Please input username:')
        password_result = getpass(prompt=f'Please input password:')
        #  pylint: disable=len-as-condition
        loginRes, mustChangePassword = apiclient.login(username_result, password_result)
        if loginRes:
            if mustChangePassword:
                print("You must change your password, connect through a web or graphical client please.")
                return False
            else:
                return loginRes
        else:
            print("Failed to login.")
        return loginRes
    else:
        print("Failed to contact this server.")
    return False

def promptForPentest():
    apiclient = APIClient.getInstance()
    pentests = apiclient.getPentestList()
    if pentests is None:
        pentests = []
    else:
        pentests.sort(key=lambda x: x["creation_date"], reverse=True)
    i = 0
    while i<=0:
        for i_p, pentest in enumerate(pentests):
            print(f"{i_p+1} : {pentest.get('nom', '')}")
        try:
            i = int(input("Selection (1-"+str(len(pentests))+"): "))
            if i <= 0 or i > len(pentests):
                print("Invalid selection, input must be between 1 and "+str(len(pentests)))
                i = 0
        except ValueError:
            i = 0
    apiclient.setCurrentPentest(pentests[i-1]["uuid"])

def parseDefaultTarget(stringToParse):
    if "|" in stringToParse:
        parts = stringToParse.split("|")
        ret = {}
        ret["check_iid"] = parts[0]
        ret["tool_iid"] = parts[1]
        ret["lvl"] = "import"
        return ret

    else:
        return {"check_iid":str(stringToParse), "lvl":"import"}


def pollup():
    """Send a file to pollenisator backend for analysis
    """
    if len(sys.argv) == 2:
        filename = sys.argv[1]
        plugin = "auto-detect"
    elif len(sys.argv) == 3:
        filename = sys.argv[1]
        plugin = sys.argv[2]
    else:
        print("Usage : pollup <filename> [plugin or auto-detect]")
        sys.exit(1)
    uploadFile(filename, plugin)

def uploadFile(filename, plugin='auto-detect'):
    apiclient = APIClient.getInstance()
    apiclient.tryConnection()
    res = apiclient.tryAuth()
    if not res:
        consoleConnect()
    print(f"INFO : Uploading results {filename}")
    msg = apiclient.importExistingResultFile(filename, plugin, parseDefaultTarget(os.environ.get("POLLENISATOR_DEFAULT_TARGET", "")), "")
    print(msg)

def pollwatch():
    from watchdog.observers import Observer
    from watchdog.events import FileSystemEventHandler
    class Handler(FileSystemEventHandler):
        @staticmethod
        def on_any_event(event):
            if event.is_directory:
                return None
            elif event.event_type == 'created':
                # Event is created, you can process it now
                uploadFile(event.src_path)
    apiclient = APIClient.getInstance()
    apiclient.tryConnection()
    res = apiclient.tryAuth()
    if not res:
        consoleConnect()
    observer = Observer()
    observer.schedule(Handler(), '.', recursive=True)
    observer.start()
    try:
        while True:
            time.sleep(5)
    finally:
        observer.stop()
        observer.join()



def main():
    """Main function. Start pollenisator application
    """

    print("""
.__    ..              ,
[__) _ || _ ._ * __ _.-+- _ ._.
|   (_)||(/,[ )|_) (_] | (_)[

""")

    import tkinter as tk
    import customtkinter
    import pollenisatorgui.core.components.utilsUI as utilsUI
    from pollenisatorgui.core.application.appli import Appli

    customtkinter.set_default_color_theme(utilsUI.getColorTheme())

    event_obj.clear()
    gc = None
    update_available = checkForUpdates()
    if update_available:
        ask_update = input("An update is available, do you want to update now (y/N)?").strip().lower() == "y"
        if ask_update:
            update()
    app = Appli()
    try:
        gc = GracefulKiller(app)
        if not app.quitting:
            app.mainloop()
        print("Exiting tkinter main loop")
    except tk.TclError:
        pass
    try:
        try:
            app.destroy()
            print("Destroying app window")
        except tk.TclError:
            pass
        if gc is not None:
            gc.kill_now = True
        event_obj.set()
        app.onClosing()
    except Exception as e:
        print("Error while closing app")
        print(e)
if __name__ == '__main__':
    main()



import subprocess
import psutil
import os
from pollenisatorgui.core.components.apiclient import APIClient
from pollenisatorgui.scripts.lan.utils import checkPath, findDc, getNICs


def main(apiclient, appli, **kwargs):
    res, path = checkPath(["ntlmrelayx.py", "ntlmrelayx"])
    if not res:
        return False, path
    APIClient.setInstance(apiclient)
    res, dc = findDc(apiclient, graphical=appli is not None)
    if not res:
        return False, dc
    try:
        device = getNICs(appli is not None)
    except ValueError as e:
        return False, str(e)
    if device is None or device.strip() == "":
        return False, "No ethernet device chosen"
    addrs = psutil.net_if_addrs()
    my_ip = addrs[device][0].address
    cmd = f"{path} -t ldap://{dc} -smb2support -wh {my_ip} -6" 
    if os.geteuid() != 0:
        cmd = "sudo "+cmd
    if appli:
        appli.launch_in_terminal(kwargs.get("default_target",None), "ntlmrelayx to ldapr", cmd, use_pollex=False)
    else:
        subprocess.run(cmd, shell=True)
    return True, f"Relaying is set up, poison the network using responder, mitm6, arp spoofing, etc."



import sys
import shlex
import multiprocessing
from pollenisatorgui.core.components.logger_config import logger

def pollex():
    if len(sys.argv) <= 1:
        print("Usage : pollex [-v] [--checkinstance <checkinstance_id> | <Plugin name> <command options> | <command to execute>]")
        sys.exit(1)
    verbose = False
    if sys.argv[1] == "-v":
        verbose = True
        execCmd = shlex.join(sys.argv[2:])
    if "--checkinstance" in sys.argv:
        try:
            index_check = sys.argv.index("--checkinstance")
            script_checkinstance_id = sys.argv[index_check+1]
            pollscript_exec(script_checkinstance_id,  verbose)
            return
        except IndexError as e:
            print("ERROR : --checkinstance option must be followed by a checkinstance id")
            sys.exit(1)
    else:
        execCmd = shlex.join(sys.argv[1:])
    pollex_exec(execCmd, verbose)

def pollscript_exec(script_checkinstance_id, verbose=False):
    import os
    import tempfile
    import time
    import shutil
    from pollenisatorgui.core.components.apiclient import APIClient
    from pollenisatorgui.core.models.checkitem import CheckItem
    from pollenisatorgui.core.models.checkinstance import CheckInstance
    from bson import ObjectId
    from pollenisatorgui.pollenisator import consoleConnect, parseDefaultTarget
    import pollenisatorgui.core.components.utils as utils
    import importlib

    apiclient = APIClient.getInstance()
    apiclient.tryConnection()
    res = apiclient.tryAuth()
    if not res:
        consoleConnect()
    check_instance = CheckInstance.fetchObject({"_id":ObjectId(script_checkinstance_id)})
    if check_instance is None:
        print("ERROR : CheckInstance not found")
        return
    script_checkitem_id = check_instance.check_iid
    check_o = CheckItem.fetchObject({"_id":ObjectId(script_checkitem_id)})
    if check_o is None:
        print("ERROR : Check not found")
        return
    if check_o.check_type != "script":
        print("ERROR : Check is not a script check")
        return
    
    script_name = os.path.normpath(check_o.title).replace(" ", "_").replace("/", "_").replace("\\", "_")+".py"
    default_target = parseDefaultTarget(os.environ.get("POLLENISATOR_DEFAULT_TARGET", ""))
    tmpdirname = tempfile.mkdtemp() ### HACK: tempfile.TemporaryDirectory() gets deleted early because a fork occurs in execute and atexit triggers.
    script_path = os.path.normpath(os.path.join(tmpdirname, script_name))
    with open(script_path, "w") as f:
        f.write(check_o.script)
    spec = importlib.util.spec_from_file_location("pollenisatorgui.scripts."+str(script_name), script_path)
    script_module = importlib.util.module_from_spec(spec)
    sys.modules["pollenisatorgui.scripts."+str(script_name)] = script_module
    spec.loader.exec_module(script_module)
    data = check_instance.getData()
    data["default_target"] = str(check_instance.getId())
    success, res = script_module.main(APIClient.getInstance(), None, **data)
    if success:
        print(f"Script {script_name} finished.\n{res}")
    else:
        print(f"Script {script_name} failed.\n{res}")

def pollex_exec(execCmd, verbose=False):
    """Send a command to execute for pollenisator-gui running instance
    """
    
    bin_name = shlex.split(execCmd)[0]
    if bin_name in ["echo", "print", "vim", "vi", "tmux", "nano", "code", "cd", "ls","pwd", "cat", "export"]:
        sys.exit(-1)
    import os
    import shutil
    import tempfile
    import time
    from pollenisatorgui.core.components.apiclient import APIClient
    from pollenisatorgui.pollenisator import consoleConnect, parseDefaultTarget
    import pollenisatorgui.core.components.utils as utils

    cmdName = os.path.splitext(os.path.basename(bin_name))[0]
    apiclient = APIClient.getInstance()
    apiclient.tryConnection()
    res = apiclient.tryAuth()
    if not res:
        consoleConnect()
    res = apiclient.getDesiredOutputForPlugin(execCmd, "auto-detect")
    (success, data) = res
    if not success:
        print(data)
        consoleConnect()
    res = apiclient.getDesiredOutputForPlugin(execCmd, "auto-detect")
    (success, data) = res
    if not success:
        print(data)
        return
    cmdName +="::"+str(time.time()).replace(" ","-")
    default_target = parseDefaultTarget(os.environ.get("POLLENISATOR_DEFAULT_TARGET", ""))
    if default_target.get("tool_iid") is not  None:
        apiclient.setToolStatus(default_target.get("tool_iid"), ["running"])
    
    if not success:
        print("ERROR : "+data)
        return
    if not data:
        print("ERROR : An error as occured : "+str(data))
        return
    local_settings = utils.load_local_settings()
    my_commands = local_settings.get("my_commands", {})
    path_to_check = set()
    bin_path = my_commands.get(bin_name, None)
    if bin_path is not None:
        path_to_check.add(bin_path)
    path_to_check.add(bin_name)
    plugin_results = data["plugin_results"]
    for plugin, plugin_data in plugin_results.items():
        if os.path.splitext(plugin)[0] in execCmd:
            path_to_check = path_to_check.union(plugin_data.get("common_bin_names", []))
    bin_path_found, result_msg = utils.checkPaths(list(path_to_check))
    if not bin_path_found:
        print("ERROR : "+result_msg)
        return
    new_bin_path = result_msg
    comm = data["command_line_options"].replace(bin_name, new_bin_path, 1)
    
    if (verbose):
        print("INFO : Matching plugins are "+str(data["plugin_results"]))
    
    tmpdirname = tempfile.mkdtemp() ### HACK: tempfile.TemporaryDirectory() gets deleted early because a fork occurs in execute and atexit triggers.
    for plugin, plugin_data in plugin_results.items():
        ext = plugin_data.get("expected_extension", ".log.txt")

        outputFilePath = os.path.join(tmpdirname, cmdName) + ext
        comm = comm.replace(f"|{plugin}.outputDir|", outputFilePath)
    if (verbose):
        print("Executing command : "+str(comm))
        print("output should be in "+str(outputFilePath))
    queue = multiprocessing.Queue()
    queueResponse = multiprocessing.Queue()
    #if comm.startswith("sudo "):
    #    returncode = utils.execute_no_fork(comm, None, True, queue, queueResponse, cwd=tmpdirname)
    #else:
    try:
        returncode = utils.execute(comm, None, queue, queueResponse, cwd=tmpdirname, printStdout=True)
    except KeyboardInterrupt:
        logger.debug("pollex KeyboardInterrupt for comm "+str(comm))
    except Exception as e:
        logger.debug("pollex Exception for comm "+str(comm)+" "+str(e))
    queue.put("kill", block=False)
    if len(plugin_results) == 1 and "Default" in plugin_results:
        if (verbose):
            print("INFO : Only default plugin found")
        response = input("No plugin matched, do you want to use default plugin to log the command and stdout ? (Y/n) :")
        if str(response).strip().lower() == "n":
            shutil.rmtree(tmpdirname)
            return
    logger.debug("pollex detect plugins "+str(plugin_results))
    atLeastOne = False
    error = ""
    for plugin, plugin_data in plugin_results.items():
        ext = plugin_data.get("expected_extension", ".log.txt")
        outputFilePath = os.path.join(tmpdirname, cmdName) + ext
        if not os.path.exists(outputFilePath):
            if os.path.exists(outputFilePath+ext):
                outputFilePath+=ext
            else:
                print(f"ERROR : Expected file was not generated {outputFilePath}")
                error = "ERROR : Expected file was not generated"
                continue
        print(f"INFO : Uploading results {outputFilePath}")
        msg = apiclient.importExistingResultFile(outputFilePath, plugin, default_target, comm)
        print(msg)
        atLeastOne = True
    if not atLeastOne:
        notes = b""
        while not queueResponse.empty():
            q = queueResponse.get(block=False)
            if q:
                if isinstance(q, str):
                    notes += q.encode()
        apiclient.setToolStatus(default_target.get("tool_iid"), ["error"], error+"\nSTDOUT:\n"+notes.decode())
    shutil.rmtree(tmpdirname)



# -*- coding: utf-8 -*-
from pollenisatorgui.core.components.apiclient import APIClient
from bson import ObjectId
from neo4j import GraphDatabase
from pollenisatorgui.scripts.lan.utils import ask_text

def mark_as_owned(uri, users, username, password):
    driver = GraphDatabase.driver(uri, auth=(username, password))
    with driver.session() as session:
        for user in users:
            account = user.get("username", "")
            if account == "":
                continue
            domain = user.get("domain", "")
            if domain == "":
                continue
            the_query = "MATCH (n) WHERE (n.name = \"{}@{}\") SET n.owned = true".format(account.strip().upper(), domain.upper())
            graph = session.run(the_query)
    driver.close()


def main(apiclient, appli, **kwargs):
    APIClient.setInstance(apiclient)
    users = apiclient.find("users", {"type":"user", "password":{"$ne":""}}, True)
    if users is None or not users:
        return False, "No owned users found"
    bloodhound_uri = ask_text(appli is not None, "Bloodhound uri:", "bolt://localhost:7687")
    if bloodhound_uri == "" or bloodhound_uri is None:
        return False, "No URI given"
    bloodhound_username = ask_text(appli is not None, "neo4j username:", "neo4j")
    if bloodhound_username is None or bloodhound_username == "":
        return False, "No username given"
    bloodhound_password = ask_text(appli is not None, "neo4j password:", "exegol4thewin", secret=True)
    if bloodhound_password is None or bloodhound_password == "":
        return False, "No password given"
    mark_as_owned(bloodhound_uri, users, bloodhound_username, bloodhound_password)
    return True, f"Marked {len(users)} users as owned in bloodhound"
